(****************************************************************************

                Copyright (c) 2007 Aleksej Kozlov

****************************************************************************)

{$DEFINE DEBUG}

{$B-,H-,I-,J-,P-,Q-,R-,S-,T-,V+,W-,X+,Z-}
{&AlignCode+,AlignData+,AlignRec-,Asm-,Cdecl-,Comments-,Delphi+,Frame+,G3+}
{&G5-,Open32-,Optimise+,OrgName-,SmartLink+,Speed+,Use32+,ZD-}
{$IFDEF DEBUG}{$D+}{&LocInfo+}{$ELSE}{$D-}{&LocInfo-}{$ENDIF}
{$M 65536}

library WPPersAB;

uses
  Os2Def, Os2Base, Os2PmApi, Strings, Dos;

const
  err_None                           = 0;
  err_NoMemory                       = 39;
  err_CannotFindRootAddressBook      = 40;
  err_CannotQueryRootAddressBookPath = 41;
  err_CannotCreateAddressBook        = 42;

const
  WPPAB_IncludeAddressBooks          = 1;
  WPPAB_IncludePersonalCards         = 2;
  WPPAB_IncludeRootAddressBook       = 4;
  WPPAB_Recursive                    = 8;

type
  PAddressBookList = ^TAddressBookList;
  TAddressBookList = packed record
    Next: PAddressBookList;
    IsAddressBook: Integer;
    IncLevel: Integer;
    DisplayName: AnsiString;
    FileName: AnsiString;
  end;

function WPPGetAddressBookList(ABPath: PChar; Flags: LongInt; var List: PAddressBookList): Integer;
var
  PtrLast: ^PAddressBookList;
  H: LHandle;
  I: Integer;
  Path: AnsiString;
  S: AnsiString;

  function IsWPP(const FName: String): Boolean;
  var
    L: Integer;
  begin
    L := Length(FName);
    if L < 4 then
      Result := False
    else
      Result := (FName[L-3] = '.') and ((FName[L-2] = 'W') or (FName[L-2] = 'w')) and ((FName[L-1] = 'P') or (FName[L-1] = 'p')) and ((FName[L] = 'P') or (FName[L] = 'p'));
  end;

  function Add(AIsAddressBook, AIncLevel: Integer; const ADisplayName, AFileName: AnsiString): Boolean;
  begin
    New(PtrLast^);
    Result := (PtrLast^ <> nil);
    if Result then
      with PtrLast^^ do
        begin
          Next := nil;
          IsAddressBook := AIsAddressBook;
          IncLevel := AIncLevel;
          DisplayName := ADisplayName;
          FileName := AFileName;
          PtrLast := @Next;
        end;
  end;

  function Scan(IncLevel: Integer; const Path: AnsiString): Boolean;
  var
    SR: SearchRec;
  begin
    Result := True;
    FindFirst(Path+'*', Directory, SR);
    while DosError = 0 do
      begin
        if (SR.Name <> '.') and (SR.Name <> '..') then
          if (SR.Attr and Directory) <> 0 then
            begin
              if (Flags and WPPAB_IncludeAddressBooks) <> 0 then
                if not Add(1, IncLevel, SR.Name, Path+SR.Name) then
                  begin
                    Result := False;
                    Break;
                  end;
              if (Flags and WPPAB_Recursive) <> 0 then
                if not Scan(IncLevel+1, Path+SR.Name+'\') then
                  begin
                    Result := False;
                    Break;
                  end;
            end
          else
            if (Flags and WPPAB_IncludePersonalCards) <> 0 then
              if IsWPP(SR.Name) then
                if not Add(0, IncLevel, Copy(SR.Name, 1, Length(SR.Name)-4), Path+SR.Name) then
                  begin
                    Result := False;
                    Break;
                  end;
        FindNext(SR);
      end;
    FindClose(SR);
  end;

begin
  List := nil;
  PtrLast := @List;
  if (Flags and (WPPAB_IncludeRootAddressBook or WPPAB_IncludeAddressBooks or WPPAB_IncludePersonalCards)) = 0 then
    begin
      Result := err_None;
      Exit;
    end;

  if (ABPath = nil) or (ABPath^ = #0) then
    begin
      H := WinQueryObject('<WP_ADDRESSBOOK>');
      if H = 0 then
        begin
          Result := err_CannotFindRootAddressBook;
          Exit;
        end;
      SetLength(Path, CCHMAXPATH+1);
      if not WinQueryObjectPath(H, @Path[1], CCHMAXPATH+1) then
        begin
          Result := err_CannotQueryRootAddressBookPath;
          Exit;
        end;
      SetLength(Path, StrLen(@Path[1]));
    end
  else
    Path := AnsiString(ABPath);

  if (Flags and WPPAB_IncludeRootAddressBook) <> 0 then
    begin
      for I := Length(Path) downto 1 do
        if Path[I] = '\' then
          Break;
      if Path[I] = '\' then
        S := Copy(Path, I+1, MaxLongInt)
      else
        S := Path;
      if not Add(1, 0, S, Path) then
        begin
          Result := err_NoMemory;
          Exit;
        end;
    end;
  if Path[Length(Path)] <> '\' then
    Path := Path+'\';
  if Scan(1, Path) then
    Result := err_None
  else
    Result := err_NoMemory;
end;

function WPPFreeAddressBookList(List: PAddressBookList): Integer;
var
  P: PAddressBookList;
begin
  while List <> nil do
    begin
      P := List;
      List := List^.Next;
      Dispose(P);
    end;
  Result := err_None;
end;

function WPPCreateAddressBook(ABPath, Title: PChar; ObjHandle: PLongInt): Integer;
const
  SetupString = 'DEFAULTVIEW=TREE;TREEVIEW=NOLINES,MINI;SHOWALLINTREEVIEW=YES;OPEN=DEFAULT;STATUSBAR=NO;SHOWPREVIEW=YES;';
var
  H: LHandle;
begin
  H := WinCreateObject('WPAddressBook', Title, SetupString, ABPath, CO_UPDATEIFEXISTS);
  if ObjHandle <> nil then
    ObjHandle^ := H;
  if H = 0 then
    Result := err_CannotCreateAddressBook
  else
    Result := err_None;
end;

exports
  WPPGetAddressBookList name 'WPPGETADDRESSBOOKLIST',
  WPPFreeAddressBookList name 'WPPFREEADDRESSBOOKLIST',
  WPPCreateAddressBook name 'WPPCREATEADDRESSBOOK';

begin
end.

